Package: encharter
Type: Package
Title: Enhanced Office Open XML Charting for 'openxlsx2'
Version: 0.9.1
Language: en-US
Authors@R: c(
    person("Jan Marvin", "Garbuszus", email = "jan.garbuszus@ruhr-uni-bochum.de", role = c("aut", "cre"))
  )
Description: Provides a high-level 'R6' interface for creating complex
    Office Open XML (OOXML) charts. Allows users to build
    multi-series combo charts with secondary axes and granular styling
    options, designed to integrate seamlessly with 'openxlsx2'.
License: MIT + file LICENSE
URL: https://janmarvin.github.io/encharter/,
        https://github.com/JanMarvin/encharter
BugReports: https://github.com/JanMarvin/encharter/issues
Encoding: UTF-8
LinkingTo: openxlsx2
Imports: R6, openxlsx2 (>= 1.26)
Suggests: testthat (>= 3.0.0), viridisLite
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-27 11:08:51 UTC; janmarvingarbuszus
Author: Jan Marvin Garbuszus [aut, cre]
Maintainer: Jan Marvin Garbuszus <jan.garbuszus@ruhr-uni-bochum.de>
Repository: CRAN
Date/Publication: 2026-06-27 11:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-07-03 00:04:58 UTC; unix
Archs: encharter.so.dSYM
