% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_find_all}
\alias{xml_find_all}
\title{Find all matches via XPath}
\usage{
xml_find_all(x, xpath)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}

\item{xpath}{Character string containing XPath expression.}
}
\value{
A pugi_nodeset (list of pugi_nodes).
}
\description{
Find all matches via XPath
}
\keyword{internal}
