% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_name}
\alias{xml_name}
\title{Get node names}
\usage{
xml_name(x)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}
}
\value{
A character vector of tag names.
}
\description{
Get node names
}
\keyword{internal}
