% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_attr}
\alias{xml_attr}
\title{Get attribute value}
\usage{
xml_attr(x, attr)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}

\item{attr}{Character string of the attribute name.}
}
\value{
A character vector of attribute values.
}
\description{
Get attribute value
}
\keyword{internal}
