% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_type}
\alias{xml_type}
\title{Get node types}
\usage{
xml_type(x)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}
}
\value{
A character vector (e.g., "element", "document").
}
\description{
Get node types
}
\keyword{internal}
