% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_add_child}
\alias{xml_add_child}
\title{Add a child node to an XML target}
\usage{
xml_add_child(.x, .name, ..., .where = -1, .value = NULL)
}
\arguments{
\item{.x}{A pugi_node, pugi_xml, or a list containing one.}

\item{.name}{The name of the new tag to create.}

\item{...}{Named arguments for attributes, unnamed for text content.}

\item{.where}{Integer; 0 to prepend, -1 to append.}

\item{.value}{Optional character string to set as text content.}
}
\value{
The newly created pugi_node.
}
\description{
Add a child node to an XML target
}
\keyword{internal}
