% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_length}
\alias{xml_length}
\title{Get count of child elements}
\usage{
xml_length(x)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}
}
\value{
An integer vector of child counts.
}
\description{
Get count of child elements
}
\keyword{internal}
