% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_remove}
\alias{xml_remove}
\title{Remove nodes from the tree}
\usage{
xml_remove(x)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}
}
\description{
Remove nodes from the tree
}
\keyword{internal}
