% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pugixml.R
\name{xml_set_attr}
\alias{xml_set_attr}
\title{Set attribute value}
\usage{
xml_set_attr(x, attr, value)
}
\arguments{
\item{x}{A pugi_node or list of nodes.}

\item{attr}{Character string of the attribute name.}

\item{value}{The value to set (coerced to character).}
}
\description{
Set attribute value
}
\keyword{internal}
